@@ -1,6 +1,5 @@
 from base64 import b64decode
 import copy
-from math import ceil
 from typing import Any, cast, Optional
 
 from algosdk.account import address_from_private_key
@@ -19,9 +18,9 @@ from algosdk.future import transaction
 from algosdk.logic import get_application_address
 from algosdk.source_map import SourceMap
 from algosdk.v2client.algod import AlgodClient
-from algosdk.constants import APP_PAGE_MAX_SIZE
 
 from beaker.application import Application, get_method_spec
+from beaker.consts import num_extra_program_pages
 from beaker.decorators import (
     HandlerFunc,
     MethodHints,
@@ -108,12 +107,8 @@ class ApplicationClient:
         assert self.clear_binary is not None and self.approval_binary is not None
 
         if extra_pages is None:
-            extra_pages = ceil(
-                (
-                    (len(self.approval_binary) + len(self.clear_binary))
-                    - APP_PAGE_MAX_SIZE
-                )
-                / APP_PAGE_MAX_SIZE
+            extra_pages = num_extra_program_pages(
+                self.approval_binary, self.clear_binary
             )
 
         sp = self.get_suggested_params(suggested_params)
@@ -456,6 +451,7 @@ class ApplicationClient:
         accounts: list[str] = None,
         foreign_apps: list[int] = None,
         foreign_assets: list[int] = None,
+        boxes: list[tuple[int, bytes]] = None,
         note: bytes = None,
         lease: bytes = None,
         rekey_to: str = None,
@@ -491,6 +487,7 @@ class ApplicationClient:
             note=note,
             lease=lease,
             rekey_to=rekey_to,
+            boxes=boxes,
             **kwargs,
         )
 
@@ -591,6 +588,7 @@ class ApplicationClient:
         accounts: list[str] = None,
         foreign_apps: list[int] = None,
         foreign_assets: list[int] = None,
+        boxes: list[tuple[int, bytes]] = None,
         note: bytes = None,
         lease: bytes = None,
         rekey_to: str = None,
@@ -611,10 +609,8 @@ class ApplicationClient:
         args = []
         for method_arg in method.args:
             name = method_arg.name
-
             if name in kwargs:
                 argument = kwargs[name]
-
                 if type(argument) is dict:
                     if hints.structs is None or name not in hints.structs:
                         raise Exception(f"Name {name} name in struct hints")
@@ -654,6 +650,7 @@ class ApplicationClient:
             accounts=accounts,
             foreign_apps=foreign_apps,
             foreign_assets=foreign_assets,
+            boxes=boxes,
             note=note,
             lease=lease,
             rekey_to=rekey_to,
@@ -716,6 +713,14 @@ class ApplicationClient:
         app_state = self.client.account_info(self.app_addr)
         return app_state
 
+    def get_box_names(self) -> list[bytes]:
+        box_resp = self.client.application_boxes(self.app_id)
+        return [b64decode(box["name"]) for box in box_resp["boxes"]]
+
+    def get_box_contents(self, name: bytes) -> bytes:
+        contents = self.client.application_box_by_name(self.app_id, name)
+        return b64decode(contents["value"])
+
     def resolve(self, to_resolve: DefaultArgument) -> Any:
         if to_resolve.resolvable_class == DefaultArgumentClass.Constant:
             return to_resolve.resolve_hint()
@@ -740,7 +745,8 @@ class ApplicationClient:
         return self.app.hints[method_name]
 
     def get_suggested_params(
-        self, sp: transaction.SuggestedParams = None
+        self,
+        sp: transaction.SuggestedParams = None,
     ) -> transaction.SuggestedParams:
 
         if sp is not None:
