@@ -1,11 +1,11 @@
 import base64
+from Cryptodome.Hash import SHA512
 from dataclasses import dataclass, field
 from typing import Optional, TYPE_CHECKING
 from pyteal import (
     Seq,
     Bytes,
     Expr,
-    Addr,
     ScratchVar,
     TealType,
     TealTypeError,
@@ -17,12 +17,15 @@ from pyteal import (
     Suffix,
     Subroutine,
     Sha512_256,
+    TxnField,
+    TxnType,
 )
 from algosdk.v2client.algod import AlgodClient
 from algosdk.source_map import SourceMap
 from algosdk.future.transaction import LogicSigAccount
 from algosdk.atomic_transaction_composer import LogicSigTransactionSigner
-from beaker.consts import PROGRAM_DOMAIN_SEPARATOR
+from beaker.consts import PROGRAM_DOMAIN_SEPARATOR, num_extra_program_pages
+from algosdk.constants import APP_PAGE_MAX_SIZE
 from beaker.lib.strings import encode_uvarint
 
 if TYPE_CHECKING:
@@ -56,6 +59,24 @@ class PrecompileTemplateValue:
     pc: int = 0
 
 
+@dataclass
+class ProgramPage:
+    #: index of the program page
+    index: int = field(kw_only=True, init=True)
+    #: binary of the page
+    _binary: bytes = field(kw_only=True, init=True)
+    #: bytes of the page as pyteal Bytes
+    binary: Bytes = field(init=False)
+    #: hash of the page in native bytes
+    _hash_digest: bytes = field(kw_only=True, init=True)
+    #: hash of the page as pyteal Addr
+    hash_digest: Bytes = field(init=False)
+
+    def __post_init__(self):
+        self.binary = Bytes(self._binary)
+        self.hash_digest = Bytes(self._hash_digest)
+
+
 class Precompile:
     """
     Precompile takes a TEAL program and handles its compilation. Used by AppPrecompile
@@ -67,12 +88,11 @@ class Precompile:
     _program_hash: Optional[str] = None
     _map: Optional[SourceMap] = None
     _template_values: list[PrecompileTemplateValue] = []
-
+    program_pages: list[ProgramPage]
     binary: Bytes = Bytes("")
 
     def __init__(self, program: str):
         self._program = program
-
         lines = self._program.splitlines()
         template_values: list[PrecompileTemplateValue] = []
         # Replace the teal program TMPL_* template variables with
@@ -106,18 +126,34 @@ class Precompile:
             # +1 to acount for the pushbytes/pushint op
             tv.pc = self._map.get_pcs_for_line(tv.line)[0] + 1
 
-    def hash(self) -> Expr:
-        """
-        address returns an expression for this Precompile.
+        def _hash_program(data: bytes) -> bytes:
+            """compute the hash"""
+            chksum = SHA512.new(truncate="256")
+            chksum.update(PROGRAM_DOMAIN_SEPARATOR.encode() + data)
+            return chksum.digest()
+
+        self.program_pages = [
+            ProgramPage(
+                index=i,
+                _binary=self._binary[i : i + APP_PAGE_MAX_SIZE],
+                _hash_digest=_hash_program(self._binary[i : i + APP_PAGE_MAX_SIZE]),
+            )
+            for i in range(0, len(self._binary), APP_PAGE_MAX_SIZE)
+        ]
+
+    def hash(self, page_idx: int = 0) -> Expr:
+        """hash returns an expression for this Precompile.  It will fail if any template_values are set.
+
+        Args:
+            page_idx(optional): If the application has multiple pages, the index of the page can be specified to get the program hash for that page.
 
-        It will fail if any template_values are set.
         """
         assert self._binary is not None
         assert len(self._template_values) == 0
         if self._program_hash is None:
             raise TealInputError("No address defined for precompile")
 
-        return Addr(self._program_hash)
+        return self.program_pages[page_idx].hash_digest
 
     def populate_template(self, *args) -> bytes:
         """
@@ -248,7 +284,7 @@ class AppPrecompile:
         self.clear: Precompile = Precompile("")
 
     def compile(self, client: AlgodClient):
-        """fully compile this lsig precompile by recursively compiling children depth first
+        """fully compile this app precompile by recursively compiling children depth first
 
         Note:
             Must be called (even indirectly) prior to using the ``approval`` and ``clear`` fields
@@ -261,13 +297,29 @@ class AppPrecompile:
         approval, clear = self.app.compile(client)
         self.approval = Precompile(approval)
         self.clear = Precompile(clear)
-
         if self.approval._binary is None:
             self.approval.assemble(client)
-
         if self.clear._binary is None:
             self.clear.assemble(client)
 
+    def get_create_config(self) -> dict[TxnField, Expr]:
+        """get a dictionary of the fields and values that should be set when creating this application that can be passed directly to the InnerTxnBuilder.Execute method"""
+        assert self.approval._binary is not None
+        assert self.clear._binary is not None
+
+        return {
+            TxnField.type_enum: TxnType.ApplicationCall,
+            TxnField.approval_program: self.approval.binary,
+            TxnField.clear_state_program: self.clear.binary,
+            TxnField.local_num_byte_slices: Int(self.app.acct_state.num_byte_slices),
+            TxnField.local_num_uints: Int(self.app.acct_state.num_uints),
+            TxnField.global_num_byte_slices: Int(self.app.app_state.num_byte_slices),
+            TxnField.global_num_uints: Int(self.app.app_state.num_uints),
+            TxnField.extra_program_pages: Int(
+                num_extra_program_pages(self.approval._binary, self.clear._binary)
+            ),
+        }
+
 
 class LSigPrecompile:
     """
