@@ -1,5 +1,6 @@
 import copy
 
+import itertools
 import pytest
 import typing
 
@@ -25,20 +26,29 @@ TOTAL_POOL_TOKENS = 10000000000
 TOTAL_ASSET_TOKENS = 10000000000
 
 
+AcctInfo = tuple[str, str, AccountTransactionSigner]
+AssertTestCase = tuple[
+    str,
+    abi.Method | decorators.HandlerFunc,
+    dict[str, typing.Any],
+    client.ApplicationClient,
+]
+
+
 @pytest.fixture(scope="session")
-def creator_acct() -> tuple[str, str, AccountTransactionSigner]:
+def creator_acct() -> AcctInfo:
     return accts[0].address, accts[0].private_key, accts[0].signer
 
 
 @pytest.fixture(scope="session")
-def user_acct() -> tuple[str, str, AccountTransactionSigner]:
+def user_acct() -> AcctInfo:
     return accts[1].address, accts[1].private_key, accts[1].signer
 
 
 @pytest.fixture(scope="session")
-def assets(creator_acct, user_acct) -> tuple[int, int]:
-    addr, sk, signer = creator_acct
-    user_addr, user_sk, user_signer = user_acct
+def assets(creator_acct: AcctInfo, user_acct: AcctInfo) -> tuple[int, int]:
+    addr, sk, _ = creator_acct
+    user_addr, _, user_signer = user_acct
 
     sp = algod_client.suggested_params()
     txns: list[transaction.Transaction] = transaction.assign_group_id(
@@ -89,7 +99,7 @@ def assets(creator_acct, user_acct) -> tuple[int, int]:
 
 
 @pytest.fixture(scope="session")
-def creator_app_client(creator_acct) -> client.ApplicationClient:
+def creator_app_client(creator_acct: AcctInfo) -> client.ApplicationClient:
     _, _, signer = creator_acct
     app = ConstantProductAMM()
     app_client = client.ApplicationClient(algod_client, app, signer=signer)
@@ -101,8 +111,8 @@ def test_app_create(creator_app_client: client.ApplicationClient):
     app_state = creator_app_client.get_application_state()
     sender = creator_app_client.get_sender()
 
-    assert (
-        app_state[ConstantProductAMM.governor.str_key()] == decode_address(sender).hex()
+    assert app_state[ConstantProductAMM.governor.str_key()] == _addr_to_hex(
+        sender
     ), "The governor should be my address"
     assert app_state[ConstantProductAMM.ratio.str_key()] == 0, "The ratio should be 0"
 
@@ -143,6 +153,10 @@ def assert_app_algo_balance(c: client.ApplicationClient, expected_algos: int):
 app_algo_balance: typing.Final = consts.algo * 10
 
 
+def build_set_governor_transaction(new_governor: str) -> dict[str, typing.Any]:
+    return {"new_governor": new_governor}
+
+
 def build_boostrap_transaction(
     app_client: client.ApplicationClient, assets: tuple[int, int]
 ) -> dict[str, typing.Any]:
@@ -236,8 +250,8 @@ def build_burn_transaction(
 def build_swap_transaction(
     app_client: ApplicationClient,
     assets: tuple[int, int],
-    pool_asset: int,
     swap_amt: int,
+    swap_asset: int = None,
 ):
     app_addr, addr, signer = (
         app_client.app_addr,
@@ -247,18 +261,57 @@ def build_swap_transaction(
 
     sp = app_client.get_suggested_params()
     a_asset, b_asset = assets
+
+    if swap_asset is None:
+        swap_asset = a_asset
+
     return {
         "suggested_params": minimum_fee_for_txn_count(sp, 2),
         "swap_xfer": TransactionWithSigner(
-            txn=transaction.AssetTransferTxn(addr, sp, app_addr, swap_amt, a_asset),
+            txn=transaction.AssetTransferTxn(addr, sp, app_addr, swap_amt, swap_asset),
             signer=signer,
         ),
-        "pool_asset": pool_asset,
         "a_asset": a_asset,
         "b_asset": b_asset,
     }
 
 
+def test_app_set_governor(
+    creator_app_client: client.ApplicationClient, user_acct: AcctInfo
+):
+    creator_addr, _ = creator_app_client.sender, creator_app_client.app_addr
+
+    user_addr, _, user_signer = user_acct
+
+    state_before = creator_app_client.get_application_state()
+
+    assert creator_addr is not None
+    assert state_before[ConstantProductAMM.governor.str_key()] == _addr_to_hex(
+        creator_addr
+    )
+
+    # Set the new gov
+    creator_app_client.call(
+        ConstantProductAMM.set_governor,
+        **build_set_governor_transaction(user_addr),
+    )
+
+    state_after = creator_app_client.get_application_state()
+    assert state_after[ConstantProductAMM.governor.str_key()] == _addr_to_hex(user_addr)
+
+    user_client = creator_app_client.prepare(signer=user_signer)
+    # Return state to old gov
+    user_client.call(
+        ConstantProductAMM.set_governor,
+        **build_set_governor_transaction(creator_addr),
+    )
+
+    state_after_revert = creator_app_client.get_application_state()
+    assert state_after_revert[ConstantProductAMM.governor.str_key()] == _addr_to_hex(
+        creator_addr
+    )
+
+
 def test_app_bootstrap(
     creator_app_client: client.ApplicationClient, assets: tuple[int, int]
 ):
@@ -442,7 +495,7 @@ def test_swap(creator_app_client: ApplicationClient):
         creator_app_client.sender,
     )
 
-    pool_asset, a_asset, b_asset = _get_tokens_from_state(creator_app_client)
+    _, a_asset, b_asset = _get_tokens_from_state(creator_app_client)
 
     assert addr
     balances_before = testing.get_balances(creator_app_client.client, [app_addr, addr])
@@ -450,9 +503,7 @@ def test_swap(creator_app_client: ApplicationClient):
     swap_amt = balances_before[addr][a_asset] // 10
     creator_app_client.call(
         ConstantProductAMM.swap,
-        **build_swap_transaction(
-            creator_app_client, (a_asset, b_asset), pool_asset, swap_amt
-        ),
+        **build_swap_transaction(creator_app_client, (a_asset, b_asset), swap_amt),
     )
 
     balances_after = testing.get_balances(creator_app_client.client, [app_addr, addr])
@@ -478,15 +529,37 @@ def test_swap(creator_app_client: ApplicationClient):
     assert ratio_after == expected_ratio
 
 
-def test_app_asserts(
+all_assert_groups = ["governor", "bootstrap", "mint", "burn", "swap"]
+
+
+@pytest.fixture(
+    scope="session",
+    params=all_assert_groups,
+)
+def grouped_assert_cases(
+    request, creator_app_client: client.ApplicationClient, user_acct: AcctInfo
+) -> list[AssertTestCase]:
+    return _assert_cases(request.param, creator_app_client, user_acct)
+
+
+@pytest.fixture(scope="session")
+def all_assert_cases(
+    creator_app_client: client.ApplicationClient, user_acct: AcctInfo
+) -> list[AssertTestCase]:
+    return _assert_cases("all", creator_app_client, user_acct)
+
+
+def _assert_cases(
+    group_key: str,
     creator_app_client: client.ApplicationClient,
-    user_acct: tuple[str, str, AccountTransactionSigner],
-):
+    user_acct: AcctInfo,
+) -> list[AssertTestCase]:
     def cases(
         m: abi.Method | decorators.HandlerFunc,
         xs: list[tuple[str, dict[str, typing.Any]]],
-    ):
-        return [(a, m, txn) for a, txn in xs]
+        client: client.ApplicationClient = creator_app_client,
+    ) -> list[AssertTestCase]:
+        return [(a, m, txn, client) for a, txn in xs]
 
     fake_addr, fake_pk, fake_signer = user_acct
     fake_client = creator_app_client.prepare(signer=fake_signer, sender=fake_addr)
@@ -500,6 +573,8 @@ def test_app_asserts(
     pool_asset, a_asset, b_asset = _get_tokens_from_state(creator_app_client)
     assets = (a_asset, b_asset)
 
+    _opt_in_to_token(fake_addr, fake_signer, pool_asset)
+
     sp = creator_app_client.client.suggested_params()
 
     def add_txn(
@@ -537,13 +612,15 @@ def test_app_asserts(
         d[key].txn.index = override
         return d
 
-    def override_sender(
-        d: dict[str, TransactionWithSigner], key: str, override: str
-    ) -> dict[str, TransactionWithSigner]:
-        print(f"{d}")
-        print(f"{d[key].txn.__dict__=}")
-        d[key].txn.sender = override
-        return d
+    def set_governor_cases():
+        def set_governor(new_gov: str):
+            return build_set_governor_transaction(new_governor=new_gov)
+
+        return cases(
+            ConstantProductAMM.set_governor,
+            [("unauthorized", set_governor(addr))],
+            fake_client,
+        )
 
     def bootstrap_cases():
         def bootstrap(
@@ -569,6 +646,8 @@ def test_app_asserts(
                     bootstrap(assets=(b_asset, b_asset)),
                 ),
             ],
+        ) + cases(
+            ConstantProductAMM.bootstrap, [("unauthorized", bootstrap())], fake_client
         )
 
     def mint_cases():
@@ -602,6 +681,10 @@ def test_app_asserts(
                     mint(pool_asset=a_asset),
                 ),
             ],
+        ) + cases(
+            ConstantProductAMM.mint,
+            [(ConstantProductAMMErrors.SenderInvalid, mint())],
+            fake_client,
         )
 
         def valid_asset_xfer(key: str):
@@ -626,9 +709,9 @@ def test_app_asserts(
                     override_axfer_amount(mint(), key, 0),
                 ),
                 (
-                    ConstantProductAMMErrors.SenderInvalid,
-                    mint(app_client=fake_client),
-                ),  # Needs to be specialized
+                    ConstantProductAMMErrors.SendAmountTooLow,
+                    override_axfer_amount(mint(), key, 1),
+                ),
             ]
 
         valid_asset_a_xfer = cases(ConstantProductAMM.mint, valid_asset_xfer("a_xfer"))
@@ -673,26 +756,104 @@ def test_app_asserts(
                     ConstantProductAMMErrors.AssetPoolIncorrect,
                     override_axfer_asset(burn(), "pool_xfer", a_asset),
                 ),
-                # (ConstantProductAMMErrors.SenderInvalid, burn(app_client=fake_client)), # Not working
             ],
+        ) + cases(
+            ConstantProductAMM.burn,
+            [(ConstantProductAMMErrors.SenderInvalid, burn())],
+            fake_client,
         )
 
         return well_formed_burn + valid_pool_xfer
 
-    # TODO: rest of them
+    def swap_cases():
+        def swap(
+            app_client: client.ApplicationClient = creator_app_client,
+            assets: tuple[int, int] = assets,
+            swap_amt: int = 1,
+            swap_asset: int = a_asset,
+        ):
+            return build_swap_transaction(app_client, assets, swap_amt, swap_asset)
+
+        well_formed_swap = cases(
+            ConstantProductAMM.swap,
+            [
+                (
+                    ConstantProductAMMErrors.AssetAIncorrect,
+                    swap(assets=(b_asset, b_asset)),
+                ),
+                (
+                    ConstantProductAMMErrors.AssetBIncorrect,
+                    swap(assets=(a_asset, a_asset)),
+                ),
+            ],
+        )
+
+        valid_swap_xfer = cases(
+            ConstantProductAMM.swap,
+            [
+                (ConstantProductAMMErrors.AmountLessThanMinimum, swap(swap_amt=0)),
+                (
+                    ConstantProductAMMErrors.SendAmountTooLow,
+                    swap(swap_amt=1),
+                ),
+                (
+                    ConstantProductAMMErrors.AssetIdsIncorrect,
+                    swap(swap_amt=0, swap_asset=pool_asset),
+                ),
+            ],
+        ) + cases(
+            ConstantProductAMM.swap,
+            [(ConstantProductAMMErrors.SenderInvalid, swap())],
+            fake_client,
+        )
+
+        return well_formed_swap + valid_swap_xfer
+
+    key_to_group: dict[str, list[AssertTestCase]] = {
+        "governor": set_governor_cases(),
+        "bootstrap": bootstrap_cases(),
+        "mint": mint_cases(),
+        "burn": burn_cases(),
+        "swap": swap_cases(),
+    }
+
+    # Sanity check - Confirm additions to `key_to_group` are added to fixture parameterization.
+    assert sorted(key_to_group.keys()) == sorted(all_assert_groups)
+
+    all_cases = list(itertools.chain.from_iterable(key_to_group.values()))
+    return all_cases if group_key == "all" else key_to_group[group_key]
+
+
+def test_approval_asserts(grouped_assert_cases: list[AssertTestCase]):
+    """
+    Confirms each logical grouping of assertions raises the expected error message.
+    """
+    for msg, method, kwargs, app_client in grouped_assert_cases:
+        with pytest.raises(LogicException, match=msg):
+            app_client.call(method, **kwargs)
+
+
+def test_approval_assert_coverage(
+    all_assert_cases: list[AssertTestCase], creator_app_client: client.ApplicationClient
+):
+    """
+    Confirms `test_approval_asserts` exercises all app approval asserts.
+
+    If `test_approval_asserts` passes and this test fails, it implies some asserts are _not_ tested.
+    """
 
     all_asserts: dict[int, ProgramAssertion] = creator_app_client.approval_asserts  # type: ignore[assignment]
-    for msg, method, kwargs in bootstrap_cases() + mint_cases() + burn_cases():
-        print(f"Testing {msg}")
+
+    for msg, method, kwargs, app_client in all_assert_cases:
         with pytest.raises(LogicException, match=msg):
             try:
-                creator_app_client.call(method, **kwargs)
+                app_client.call(method, **kwargs)
             except LogicException as e:
                 if e.pc in all_asserts:
                     del all_asserts[e.pc]
                 raise e
 
-    print(f"Unhandled asserts ({len(all_asserts)}): {all_asserts}")
+    assert len(all_asserts) == 0
 
 
 def _get_tokens_to_mint(
@@ -748,3 +909,7 @@ def _opt_in_to_token(addr: str, signer: AccountTransactionSigner, id: int):
         )
     )
     atc.execute(algod_client, 2)
+
+
+def _addr_to_hex(addr: str) -> str:
+    return decode_address(addr).hex()
